@@ -1,23 +1,27 @@
 use crate::interactive::{
     widgets::{
-        Entries, EntriesProps, Footer, FooterProps, Header, HelpPane, HelpPaneProps, MarkPane,
-        MarkPaneProps, COLOR_MARKED,
+        Entries, EntriesProps, Footer, FooterProps, GlobPane, GlobPaneProps, Header, HelpPane,
+        HelpPaneProps, MarkPane, MarkPaneProps, COLOR_MARKED,
     },
-    AppState, DisplayOptions, FocussedPane,
+    AppState, Cursor, DisplayOptions, FocussedPane,
 };
-use dua::traverse::Traversal;
 use std::borrow::Borrow;
+use tui::backend::Backend;
 use tui::{
-    buffer::Buffer,
     layout::{Constraint, Direction, Layout, Rect},
     style::Modifier,
     style::{Color, Style},
 };
+use tui_react::Terminal;
 use Constraint::*;
 use FocussedPane::*;
 
 pub struct MainWindowProps<'a> {
-    pub traversal: &'a Traversal,
+    pub current_path: String,
+    pub entries_traversed: u64,
+    pub total_bytes: Option<u128>,
+    pub start: std::time::Instant,
+    pub elapsed: Option<std::time::Duration>,
     pub display: DisplayOptions,
     pub state: &'a AppState,
 }
@@ -27,34 +31,34 @@ pub struct MainWindow {
     pub help_pane: Option<HelpPane>,
     pub entries_pane: Entries,
     pub mark_pane: Option<MarkPane>,
+    pub glob_pane: Option<GlobPane>,
 }
 
 impl MainWindow {
-    pub fn render<'a>(
+    pub fn render<'a, B>(
         &mut self,
         props: impl Borrow<MainWindowProps<'a>>,
         area: Rect,
-        buf: &mut Buffer,
-    ) {
+        terminal: &mut Terminal<B>,
+        cursor: &mut Cursor,
+    ) where
+        B: Backend,
+    {
         let MainWindowProps {
-            traversal:
-                Traversal {
-                    tree,
-                    entries_traversed,
-                    total_bytes,
-                    start,
-                    elapsed,
-                    ..
-                },
+            current_path,
+            entries_traversed,
+            total_bytes,
+            start,
+            elapsed,
             display,
             state,
         } = props.borrow();
 
-        let (entries_style, help_style, mark_style) = pane_border_style(state.focussed);
+        let (entries_style, help_style, mark_style, glob_style) = pane_border_style(state.focussed);
         let (header_area, content_area, footer_area) = main_window_layout(area);
 
         let header_bg_color = header_background_color(self.is_anything_marked(), state.focussed);
-        Header.render(header_bg_color, header_area, buf);
+        Header.render(header_bg_color, header_area, terminal.current_buffer_mut());
 
         let (entries_area, help_pane, mark_pane) = {
             let (left_pane, right_pane) = content_layout(content_area);
@@ -69,12 +73,23 @@ impl MainWindow {
             }
         };
 
+        let (entries_area, glob_pane) = match &mut self.glob_pane {
+            Some(ref mut glob_pane) => {
+                let regions = Layout::default()
+                    .direction(Direction::Vertical)
+                    .constraints([Max(256), Length(3)].as_ref())
+                    .split(entries_area);
+                (regions[0], Some((regions[1], glob_pane)))
+            }
+            None => (entries_area, None),
+        };
+
         if let Some((mark_area, pane)) = mark_pane {
             let props = MarkPaneProps {
                 border_style: mark_style,
                 format: display.byte_format,
             };
-            pane.render(props, mark_area, buf);
+            pane.render(props, mark_area, terminal.current_buffer_mut());
         }
 
         if let Some((help_area, pane)) = help_pane {
@@ -82,22 +97,30 @@ impl MainWindow {
                 border_style: help_style,
                 has_focus: matches!(state.focussed, Help),
             };
-            pane.render(props, help_area, buf);
+            pane.render(props, help_area, terminal.current_buffer_mut());
         }
 
         let marked = self.mark_pane.as_ref().map(|p| p.marked());
         let props = EntriesProps {
-            tree,
-            root: state.root,
+            current_path: current_path.clone(),
             display: *display,
             entries: &state.entries,
             marked,
-            selected: state.selected,
+            selected: state.navigation().selected,
             border_style: entries_style,
             is_focussed: matches!(state.focussed, Main),
             sort_mode: state.sorting,
         };
-        self.entries_pane.render(props, entries_area, buf);
+        self.entries_pane
+            .render(props, entries_area, terminal.current_buffer_mut());
+
+        if let Some((glob_area, pane)) = glob_pane {
+            let props = GlobPaneProps {
+                border_style: glob_style,
+                has_focus: matches!(state.focussed, Glob),
+            };
+            pane.render(props, glob_area, terminal, cursor);
+        }
 
         Footer.render(
             FooterProps {
@@ -110,7 +133,7 @@ impl MainWindow {
                 sort_mode: state.sorting,
             },
             footer_area,
-            buf,
+            terminal.current_buffer_mut(),
         );
     }
 
@@ -154,7 +177,7 @@ fn main_window_layout(area: Rect) -> (Rect, Rect, Rect) {
     (regions[0], regions[1], regions[2])
 }
 
-fn pane_border_style(focused_pane: FocussedPane) -> (Style, Style, Style) {
+fn pane_border_style(focused_pane: FocussedPane) -> (Style, Style, Style, Style) {
     let grey = Style {
         fg: Color::DarkGray.into(),
         bg: Color::Reset.into(),
@@ -163,8 +186,9 @@ fn pane_border_style(focused_pane: FocussedPane) -> (Style, Style, Style) {
     };
     let bold = Style::default().add_modifier(Modifier::BOLD);
     match focused_pane {
-        Main => (bold, grey, grey),
-        Help => (grey, bold, grey),
-        Mark => (grey, grey, bold),
+        Main => (bold, grey, grey, grey),
+        Help => (grey, bold, grey, grey),
+        Mark => (grey, grey, bold, grey),
+        Glob => (grey, grey, grey, bold),
     }
 }
